from typing import Optional, Type, get_origin, get_args


def is_generic_type(annotation: Optional[Type[object]]) -> bool:
    return get_origin(annotation) is not None and len(get_args(annotation)) > 0



from typing import Dict, List, Union, Any

Field = Dict[str, Union[str, List[str]]]
# Just what we get from `from fastavro import parse_schema`
FastAvroParsedSchema = Union[str, List[Any], Dict[Any, Any]]



import json
from pathlib import Path
from typing import List, Optional, Dict

import pytest
from pydantic import BaseModel, UUID4, ValidationError
from pydantic_settings import BaseSettings

from wunderkafka.serdes.avromodel import derive


class NestedModel(BaseModel):
    id: UUID4


def test_simple_base_model() -> None:
    schema = derive(NestedModel, topic='test_data_1')
    assert json.loads(schema) == {
        'type': 'record',
        'name': 'test_data_1_value',
        'fields': [
            {
                'name': 'id',
                'type': {
                    'type': 'string',
                    'logicalType': 'uuid'
                }
            }
        ]
    }


class ResultModel(BaseModel):
    id: UUID4
    nested: List[NestedModel]


def test_nested_base_model_list() -> None:
    schema = derive(ResultModel, topic='test_data_1')
    assert json.loads(schema) == {
        'type': 'record',
        'name': 'test_data_1_value',
        'fields': [
            {
                'name': 'id',
                'type': {
                    'type': 'string',
                    'logicalType': 'uuid'
                }
            },
            {
                'name': 'nested',
                'type': {
                    'type': 'array',
                    'items': {
                        'type': 'record',
                        'name': 'NestedModel',
                        'fields': [
                            {
                                'name': 'id',
                                'type': {
                                    'type': 'string',
                                    'logicalType': 'uuid'
                                }
                            }
                        ]
                    },
                    'name': 'nested'
                }
            }
        ]
    }


class HasDictStrSchema(BaseModel):
    dct: Dict[str, NestedModel]


def test_nested_base_model_dict_str_key() -> None:
    schema = derive(HasDictStrSchema, topic='test_data_1')
    assert json.loads(schema) == {
        'type': 'record',
        'name': 'test_data_1_value',
        'fields': [
            {
                'name': 'dct',
                'type': {
                    'type': 'map',
                    'values': {
                        'type': 'record',
                        'name': 'NestedModel',
                        'fields': [
                            {
                                'name': 'id',
                                'type': {
                                    'type': 'string',
                                    'logicalType': 'uuid'
                                }
                            }
                        ]
                    },
                    'name': 'dct'
                }
            }
        ]
    }


class HasDictIntSchema(BaseModel):
    dct: Dict[int, NestedModel]


def test_nested_base_model_dict_int_key() -> None:
    # InvalidMap derived from Exception =/
    with pytest.raises(Exception):
        derive(HasDictIntSchema, topic='test_data_1')


class Error(BaseModel):
    message: str


class ImageNestedModel(BaseModel):
    id: UUID4
    nested: Optional[NestedModel]
    error: Optional[Error]


def test_nested_base_model_optional() -> None:
    schema = derive(ImageNestedModel, topic='test_data_1')
    assert json.loads(schema) == {
        'type': 'record',
        'name': 'test_data_1_value',
        'fields': [
            {
                'name': 'id',
                'type': {
                    'type': 'string',
                    'logicalType': 'uuid'
                }
            },
            {
                'name': 'nested',
                'type': [
                    {
                        'type': 'record',
                        'name': 'NestedModel',
                        'fields': [
                            {
                                'name': 'id',
                                'type': {
                                    'type': 'string',
                                    'logicalType': 'uuid'
                                }
                            }
                        ]
                    },
                    'null'
                ]
            },
            {
                'name': 'error',
                'type': [
                    {
                        'type': 'record',
                        'name': 'Error',
                        'fields': [
                            {
                                'name': 'message',
                                'type': 'string'
                            }
                        ]
                    },
                    'null'
                ]
            }
        ]
    }


class DeepNestedModel(BaseModel):
    result: ResultModel


def test_deep_nested_base_model() -> None:
    schema = derive(DeepNestedModel, topic='test_data_1')
    assert json.loads(schema) == {
        'type': 'record',
        'name': 'test_data_1_value',
        'fields': [
            {
                'name': 'result',
                'type': {
                    'type': 'record',
                    'name': 'ResultModel',
                    'fields': [
                        {
                            'name': 'id',
                            'type': {
                                'type': 'string',
                                'logicalType': 'uuid'
                            }
                        },
                        {
                            'name': 'nested',
                            'type': {
                                'type': 'array',
                                'items': {
                                    'type': 'record',
                                    'name': 'NestedModel',
                                    'fields': [
                                        {
                                            'name': 'id',
                                            'type': {
                                                'type': 'string',
                                                'logicalType': 'uuid'
                                            }
                                        }
                                    ]
                                },
                                'name': 'nested'
                            }
                        }
                    ]
                }
            }
        ]
    }


class MLConfig(BaseSettings):
    weights: Path
    threshold: float = 0.5


class MetricV22(BaseSettings):
    line_speed: Optional[int]
    defect_detected: Optional[bool] = False
    model_on: Optional[bool] = False
    squad_number: int = 0
    model_config: MLConfig                                                               # type: ignore[assignment,misc]

    class Config:
        extra = 'allow'


def test_pydantic_base_settings_v22_with_defaults_complicated_field() -> None:
    with pytest.raises(ValueError):
        derive(MetricV22, topic='some_topic')

    with pytest.raises(ValidationError):
        MetricV22(line_speed=2, model_config=MLConfig(path='test'))  # type: ignore



from typing import List
from pathlib import Path

import pytest

from tests.integration.confluent.conftest import Msg
from wunderkafka.consumers.types import StreamResult
from wunderkafka.serdes.headers import ConfluentClouderaHeadersHandler
from wunderkafka.tests import TestConsumer, TestHTTPClient
from wunderkafka.serdes.avro import FastAvroDeserializer
from wunderkafka.tests.consumer import Message
from wunderkafka.schema_registry import SimpleCache, ConfluentSRClient
from wunderkafka.consumers.constructor import HighLevelDeserializingConsumer


SIGNAL_MESSAGE = Msg(
    payload=b'\x08test\x0cstring\x0cstring\x04NA\xcc\xb8\xeb\xa6\x80_',
    deserialized={
        'source': 'test',
        'type': 'string',
        'id': 'string',
        'value': 'NA',
        'ts': 1632128298534,
    },
)


HEADERS = (
    b'\x00\x00\x00\x08<',
)


@pytest.mark.parametrize("header", list(HEADERS))
def test_consume_moving_parts(sr_root_existing: Path, topic: str, header: bytes) -> None:
    consumer = HighLevelDeserializingConsumer(
        consumer=TestConsumer([Message(topic, value=SIGNAL_MESSAGE.serialized(header))]),
        schema_registry=ConfluentSRClient(TestHTTPClient(sr_root_existing), SimpleCache()),
        headers_handler=ConfluentClouderaHeadersHandler().parse,
        deserializer=FastAvroDeserializer(),
        stream_result=True,
    )

    consumer.subscribe([topic], from_beginning=True)

    events: List[StreamResult] = consumer.consume()
    [event] = events

    assert event.payload == SIGNAL_MESSAGE.deserialized



from dataclasses import dataclass
from pathlib import Path
from typing import Dict, Any

import pytest


@pytest.fixture
def topic() -> str:
    return 'testing_wunderkafka'


@pytest.fixture
def sr_root(fixtures_root: Path) -> Path:
    return fixtures_root / 'schema_registry' / 'confluent'


@pytest.fixture
def sr_root_create(sr_root: Path) -> Path:
    return sr_root / 'create'


@pytest.fixture
def sr_root_existing(sr_root: Path) -> Path:
    return sr_root / 'existing'


@pytest.fixture
def sr_root_update(sr_root: Path) -> Path:
    return sr_root / 'update'


@dataclass
class Msg(object):
    payload: bytes
    deserialized: Dict[str, Any]

    def serialized(self, header: bytes) -> bytes:
        return header + self.payload



from typing import Optional
from pathlib import Path

from pydantic import BaseModel, UUID4

from wunderkafka.serdes.string.serializers import StringSerializer


from wunderkafka.tests import TestProducer
from wunderkafka.producers.constructor import HighLevelSerializingProducer


class Image(BaseModel):
    id: Optional[UUID4] = None
    path: Optional[str] = None


def test_string_producer_string_key_no_schema(sr_root_existing: Path, topic: str) -> None:
    test_producer = TestProducer()
    producer = HighLevelSerializingProducer(
        producer=test_producer,
        schema_registry=None,
        header_packer=None,
        value_serializer=StringSerializer(),
        key_serializer=StringSerializer(),
        mapping={topic: (str, str)},
        protocol_id=0,
    )

    key = "714fc713-37ff-4477-9157-cb4f14b63e1a"
    value = "/var/folders/x5/zlpmj3915pqfj5lhnlq5qwkm0000gn/T/tmprq2rktq3"

    producer.send_message(topic, value, key)

    [message] = test_producer.sent

    assert message.key == b'714fc713-37ff-4477-9157-cb4f14b63e1a'
    assert message.value == b'/var/folders/x5/zlpmj3915pqfj5lhnlq5qwkm0000gn/T/tmprq2rktq3'



from typing import Optional
from pathlib import Path
from uuid import UUID

from pydantic import BaseModel, UUID4

from wunderkafka.serdes.avromodel.serializers import AvroModelSerializer
from wunderkafka.serdes.store import AvroModelRepo

from wunderkafka.serdes.string.serializers import StringSerializer
from wunderkafka.serdes.headers import ConfluentClouderaHeadersHandler
from wunderkafka.tests import TestProducer, TestHTTPClient
from wunderkafka.schema_registry import SimpleCache, ConfluentSRClient
from wunderkafka.producers.constructor import HighLevelSerializingProducer


class Image(BaseModel):
    id: UUID4
    path: Optional[str] = None


def test_avro_producer_string_key_create_schema(sr_root_existing: Path) -> None:
    topic = 'testing_avro_str_producer'
    test_producer = TestProducer()
    sr_client = ConfluentSRClient(TestHTTPClient(sr_root_existing), SimpleCache())
    producer = HighLevelSerializingProducer(
        producer=test_producer,
        schema_registry=sr_client,
        header_packer=ConfluentClouderaHeadersHandler().pack,
        value_serializer=AvroModelSerializer(AvroModelRepo()),
        key_serializer=StringSerializer(),
        mapping={topic: (Image, str)},
        protocol_id=0,
    )

    key = "714fc713-37ff-4477-9157-cb4f14b63e1a"
    value = Image(
        id=UUID("714fc713-37ff-4477-9157-cb4f14b63e1a"),
        path="/var/folders/x5/zlpmj3915pqfj5lhnlq5qwkm0000gn/T/tmprq2rktq3",
    )

    producer.send_message(topic, value, key)

    [message] = test_producer.sent

    assert message.key == b'714fc713-37ff-4477-9157-cb4f14b63e1a'
    assert message.value == b'\x00\x00\x00\x00\x13H714fc713-37ff-4477-9157-cb4f14b63e1a\x02x/var/folders/x5/zlpmj3915pqfj5lhnlq5qwkm0000gn/T/tmprq2rktq3'  # noqa: E501



import json
from typing import List

from dataclasses_avroschema.pydantic import AvroBaseModel
from pydantic import UUID4

from wunderkafka.serdes.avromodel import derive


class AvroNestedClass(AvroBaseModel):
    id: UUID4


class AvroResultClass(AvroBaseModel):
    id: UUID4
    nested: List[AvroNestedClass]


class AvroSimilarImage(AvroBaseModel):
    id: UUID4


def test_avro_model() -> None:
    schema = derive(AvroSimilarImage, topic='test_data_1')
    assert json.loads(schema) == {
        'type': 'record',
        'name': 'test_data_1_value',
        'fields': [
            {
                'name': 'id',
                'type': {
                    'type': 'string',
                    'logicalType': 'uuid'
                }
            }
        ]
    }


def test_avro_base_model() -> None:
    schema = derive(AvroResultClass, topic='test_data_1')
    assert json.loads(schema) == {
        'type': 'record',
        'name': 'test_data_1_value',
        'fields': [
            {
                'name': 'id',
                'type': {
                    'type': 'string',
                    'logicalType': 'uuid'
                }
            },
            {
                'name': 'nested',
                'type': {
                    'type': 'array',
                    'items': {
                        'type': 'record',
                        'name': 'AvroNestedClass',
                        'fields': [
                            {
                                'name': 'id',
                                'type': {
                                    'type': 'string',
                                    'logicalType': 'uuid'
                                }
                            }
                        ]
                    },
                    'name': 'nested'
                }
            }
        ]
    }



import json
import sys
import time
from typing import Optional, Union, List
from datetime import datetime
from dataclasses import dataclass

import pytest
from pydantic import Field, BaseModel, ValidationError, ConfigDict, UUID4
from pydantic_settings import BaseSettings

from dataclasses_avroschema import AvroModel
from wunderkafka.serdes.avromodel import derive


# ToDo (tribunsky.kir): review some tests. As pydantic V2 changes it's behaviour, some tests are useless:
#                       we still can derive correct model, but we will be unable to actually populate it in runtime.
# ToDo (tribunsky.kir): switch to derivation of dataclasses-avroschema[pydantic], when it start support v2:
#                       https://github.com/marcosschroh/dataclasses-avroschema/issues/381


@dataclass
class SomeData(AvroModel):
    field1: int
    field2: str


@dataclass
class SomeDefaultData(AvroModel):
    field1: Optional[int] = None
    field2: Optional[str] = None


class Metrics(BaseModel):
    line_speed: Optional[int]
    defect_detected: Optional[bool] = False
    model_on: Optional[bool] = False
    squad_number: int = 0


class Metric(BaseSettings):
    line_speed: Optional[int]
    defect_detected: Optional[bool] = False
    model_on: Optional[bool] = False
    squad_number: int = 0


class MetricV2(BaseSettings):
    line_speed: Optional[int]
    defect_detected: Optional[bool] = False
    model_on: Optional[bool] = False
    squad_number: int = 0
    model_config: str  # type: ignore


class MetricV21(BaseModel):
    line_speed: Optional[int]
    defect_detected: Optional[bool] = False
    model_on: Optional[bool] = False
    squad_number: int = 0
    model_config: str  # type: ignore[assignment,misc]

    class Config:
        extra = 'allow'


class Event(BaseModel):
    id: Optional[int]
    ts: Optional[int] = None

    class Meta:
        namespace = "any.data"


class Mixed(BaseModel):
    text: str
    value: float
    integer: int = 0
    string: str = 'str'


# Pydantic allows following of non-default arguments, but dataclasses are not.
class ImageData(BaseModel):
    name: str = 'str'
    image: bytes
    camera: str
    ts: float = Field(default_factory=time.time)


class OtherImageData(BaseModel):
    image: bytes
    camera: str
    ts: float = Field(default_factory=time.time)
    name: str = 'str'


class TsWithMeta(BaseModel):
    ts: datetime = Field(default_factory=datetime.now)

    class Meta:
        namespace = 'com.namespace.my'
        name = 'MsgKey'


class User1(BaseModel):
    model_config = ConfigDict(extra='ignore')
    name: str


class User2(BaseModel):
    model_config: ConfigDict = ConfigDict(extra='ignore')  # type: ignore[assignment,misc]
    name: str


class User3(BaseModel):
    model_config: int = ConfigDict(extra='ignore')  # type: ignore[assignment,misc]
    name: str


class Image(BaseModel):
    id: Optional[UUID4] = None
    path: Optional[str] = None


class Item(BaseModel):
    id: UUID4


class ItemList(BaseModel):
    items: Optional[List[Item]] = None


def test_dataclass() -> None:
    schema = derive(SomeData, topic='test_data_1')
    assert json.loads(schema) == {
        'type': 'record',
        'name': 'test_data_1_value',
        'fields': [
            {
                'name': 'field1',
                'type': 'long',
            },
            {
                'name': 'field2',
                'type': 'string',
            },
        ],
    }


def test_dataclass_defaults() -> None:
    schema = derive(SomeDefaultData, topic='test_data_2')

    assert json.loads(schema) == {
        'type': 'record',
        'name': 'test_data_2_value',
        'fields': [
            {
                'name': 'field1',
                'type': ['null', 'long'],
                'default': None,
            },
            {
                'name': 'field2',
                'type': ['null', 'string'],
                'default': None,
            },
        ],
    }


def test_pydantic_with_defaults() -> None:
    schema = derive(Metrics, topic='some_topic')

    assert json.loads(schema) == {
        "type": "record",
        "name": "some_topic_value",
        "fields": [
            {
                "name": "line_speed",
                "type": [
                    "long",
                    "null"
                ]
            },
            {
                "name": "defect_detected",
                "type": [
                    "boolean",
                    "null"
                ],
                "default": False
            },
            {
                "name": "model_on",
                "type": [
                    "boolean",
                    "null"
                ],
                "default": False
            },
            {
                "name": "squad_number",
                "type": "long",
                "default": 0,
            }
        ]
    }


def test_pydantic_defaults() -> None:
    schema = derive(Mixed, topic='topic')

    assert json.loads(schema) == {
        'type': 'record',
        'name': 'topic_value',
        'fields': [
            {
                'type': 'string',
                'name': 'text',
            },
            {
                'type': 'double',
                'name': 'value',
            },
            {
                'type': 'long',
                'name': 'integer',
                'default': 0,
            },
            {
                'type': 'string',
                'name': 'string',
                'default': 'str'
            },
        ],
    }


def test_pydantic_defaults_none() -> None:
    schema = derive(Event, topic='topic')

    assert json.loads(schema) == {
        'type': 'record',
        'name': 'topic_value',
        'namespace': 'any.data',
        'fields': [
            {
                'type': ['long', 'null'],
                'name': 'id',
            },
            {
                'type': ['null', 'long'],
                'name': 'ts',
                'default': None,
            },
        ],
    }


def test_pydantic_mixed_defaults() -> None:
    s1 = derive(ImageData, topic='topic')
    s2 = derive(OtherImageData, topic='topic')
    assert json.loads(s1) == {
        'type': 'record',
        'name': 'topic_value',
        'fields': [
            {
                'name': 'name',
                'type': 'string',
                'default': 'str'
            },
            {
                'name': 'image',
                'type': 'bytes',
            },
            {
                'name': 'camera',
                'type': 'string',
            },
            {
                'name': 'ts',
                'type': 'double',
            },
        ],
    }
    assert json.loads(s1) != json.loads(s2)


def test_pydantic_with_meta() -> None:
    schema = derive(TsWithMeta, topic='topic')

    assert json.loads(schema) == {
        'type': 'record',
        'namespace': 'com.namespace.my',
        'name': 'MsgKey',
        'fields': [
            {
                'name': 'ts',
                'type': {
                    'logicalType': 'timestamp-millis',
                    'type': 'long',
                }
            }
        ],
    }


def test_pydantic_base_settings_with_defaults() -> None:
    schema = derive(Metric, topic='some_topic')

    assert json.loads(schema) == {
        "type": "record",
        "name": "some_topic_value",
        "fields": [
            {
                "name": "line_speed",
                "type": [
                    "long",
                    "null"
                ]
            },
            {
                "name": "defect_detected",
                "type": [
                    "boolean",
                    "null"
                ],
                "default": False
            },
            {
                "name": "model_on",
                "type": [
                    "boolean",
                    "null"
                ],
                "default": False
            },
            {
                "name": "squad_number",
                "type": "long",
                "default": 0,
            }
        ]
    }


def test_pydantic_base_settings_v2_with_defaults() -> None:
    with pytest.raises(ValueError):
        derive(MetricV2, topic='some_topic')

    with pytest.raises(ValidationError):
        MetricV2(line_speed=2, model_config='str')  # type: ignore[call-arg]


def test_pydantic_base_settings_v21_with_defaults() -> None:
    with pytest.raises(ValueError):
        derive(MetricV21, topic='some_topic')

    # https://github.com/pydantic/pydantic/issues/8469
    MetricV21(line_speed=2, model_config='str')  # type: ignore


if sys.version_info <= (3, 10):
    class ParentOptional(BaseModel):
        volume: float = Field(description='...')
        weight: Optional[float] = Field(description='...')


    class ParentUnion(BaseModel):
        payload: Union[bytes, str]
        value: float
else:
    class ParentOptional(BaseModel):
        volume: float = Field(description='...')
        weight: float | None = Field(description='...')


    class ParentUnion(BaseModel):
        payload: bytes | str
        value: float


def test_optional_type() -> None:
    schema = derive(ParentOptional, topic='test_data_1')
    assert json.loads(schema) == {
        'type': 'record',
        'name': 'test_data_1_value',
        'fields': [
            {
                'name': 'volume',
                'type': 'double',
            },
            {
                'name': 'weight',
                'type': ['double', 'null'],
            },
        ],
    }


def test_union_type() -> None:
    schema = derive(ParentUnion, topic='test_data_1')
    assert json.loads(schema) == {
        'type': 'record',
        'name': 'test_data_1_value',
        'fields': [
            {
                'name': 'payload',
                'type': ['bytes', 'string'],
            },
            {
                'name': 'value',
                'type': 'double',
            },
        ],
    }


def test_pydantic_v2_legal_model_config() -> None:
    schema = derive(User1, topic='test_data_1')
    assert json.loads(schema) == {
        'type': 'record',
        'name': 'test_data_1_value',
        'fields': [
            {
                'name': 'name',
                'type': 'string',
            },
        ],
    }


def test_pydantic_v2_legal_model_config_annotated() -> None:
    schema = derive(User2, topic='test_data_1')
    assert json.loads(schema) == {
        'type': 'record',
        'name': 'test_data_1_value',
        'fields': [
            {
                'name': 'name',
                'type': 'string',
            },
        ],
    }


def test_pydantic_v2_wrong_model_config_annotated() -> None:
    # Maybe we should not raise an error here and check if field value, not the annotation, is ConfigDict?
    with pytest.raises(ValueError):
        derive(User3, topic='test_data_1')


def test_pydantic_annotated() -> None:
    schema = derive(Image, topic='test_data_1')
    assert json.loads(schema) == {
        "type": "record",
        "name": "test_data_1_value",
        "fields": [
            {
                "name": "id",
                "type": [
                    "null",
                    {
                        "type": "string",
                        "logicalType": "uuid",
                    }
                ],
                "default": None
            },
            {
                "name": "path",
                "type": [
                    "null",
                    "string"
                ],
                "default": None
            }
        ]
    }


def test_pydantic_annotated_items() -> None:
    schema = derive(ItemList, topic='test_data_1')
    assert json.loads(schema) == {
        "type": "record",
        "name": "test_data_1_value",
        "fields": [
            {
                "name": "items",
                "type": [
                    "null",
                    {
                        "type": "array",
                        "items": {
                            "type": "record",
                            "name": "Item",
                            "fields": [
                                {
                                    "name": "id",
                                    "type": {
                                        "type": "string",
                                        "logicalType": "uuid",
                                    }
                                }
                            ]
                        },
                        "name": "item"
                    }
                ],
                "default": None
            }
        ]
    }



import json

from pydantic import BaseModel

from wunderkafka.serdes.avromodel import derive


class MyModel(BaseModel):
    a: int
    b: str


def test_derive_hyphens(topic: str = 'my-topic') -> None:
    schema = derive(MyModel, topic=topic)
    assert json.loads(schema) == {
        'name': 'my_topic_value',
        'type': 'record',
        'fields': [
            {
                'name': 'a',
                'type': 'long'
            },
            {
                'name': 'b',
                'type': 'string'
            },
        ],
    }


def test_derive_leading_underscores(topic: str = '_my-topic') -> None:
    schema = derive(MyModel, topic=topic)
    assert json.loads(schema) == {
        'name': 'my_topic_value',
        'type': 'record',
        'fields': [
            {
                'name': 'a',
                'type': 'long'
            },
            {
                'name': 'b',
                'type': 'string'
            },
        ],
    }



from pathlib import Path

import pytest


from wunderkafka.serdes.json import HAS_JSON_SCHEMA

if not HAS_JSON_SCHEMA:
    pytest.skip("skipping json-schema-only tests", allow_module_level=True)
from wunderkafka.serdes.store import JSONRepo
from wunderkafka.serdes.json.serializers import JSONSerializer

from wunderkafka.serdes.headers import ConfluentClouderaHeadersHandler
from wunderkafka.tests import TestProducer, TestHTTPClient
from wunderkafka.schema_registry import SimpleCache, ConfluentSRClient
from wunderkafka.producers.constructor import HighLevelSerializingProducer


def test_json_producer_create_schema(sr_root_existing: Path) -> None:
    topic = 'testing_json_str_producer'
    schema_str = '{"properties": {"id": {"anyOf": [{"format": "uuid4", "type": "string"}, {"type": "null"}], "default": null, "title": "Id"}, "path": {"anyOf": [{"type": "string"}, {"type": "null"}], "default": null, "title": "Path"}}, "title": "Image", "type": "object", "additionalProperties": false}'  # noqa: E501
    test_producer = TestProducer()
    sr_client = ConfluentSRClient(TestHTTPClient(sr_root_existing), SimpleCache())
    producer = HighLevelSerializingProducer(
        producer=test_producer,
        schema_registry=sr_client,
        header_packer=ConfluentClouderaHeadersHandler().pack,
        serializer=JSONSerializer(sr_client.client),
        store=JSONRepo(),
        mapping={topic: schema_str},
        protocol_id=0,
    )

    key = None
    value = {
        "id": "714fc713-37ff-4477-9157-cb4f14b63e1a",
        "path": "/var/folders/x5/zlpmj3915pqfj5lhnlq5qwkm0000gn/T/tmprq2rktq3",
    }

    producer.send_message(topic, value, key)

    [message] = test_producer.sent

    assert message.key is None
    assert message.value == b'\x00\x00\x00\x07<{"id": "714fc713-37ff-4477-9157-cb4f14b63e1a", "path": "/var/folders/x5/zlpmj3915pqfj5lhnlq5qwkm0000gn/T/tmprq2rktq3"}'  # noqa: E501



import inspect
import sys
import typing
from types import MappingProxyType
from typing import Any, List, Type, Union, get_origin

# We check it via import to avoid using nested imports in implementation in `is_union_type() function`
HAS_UNION_TYPE = True
try:
    from types import UnionType  # type: ignore[attr-defined]
except ImportError:
    HAS_UNION_TYPE = False

if sys.version_info >= (3, 9):
    from typing import Annotated
else:
    from typing_extensions import Annotated


def create_annotation(generic: Any, types_list: List[Type[object]]) -> Type[object]:
    # return generic[Union[types_list]]
    return generic[tuple(types_list)]


# Same as `sys.version_info <= (3, 10)`
if HAS_UNION_TYPE is False:
    def is_union_type(generic: Any) -> bool:
        return False
else:
    def is_union_type(generic: Any) -> bool:
        return inspect.isclass(generic) and issubclass(generic, UnionType)

if sys.version_info >= (3, 9):
    def get_generic(annotation: Any) -> Any:
        return get_origin(annotation)

    def is_annotated_type(annotation: Any) -> bool:
        return get_origin(annotation) is Annotated
else:
    _TYPE_MAPPING = MappingProxyType({
        getattr(t, '__origin__', None): t for t in typing.__dict__.values() if hasattr(t, '__origin__')
    })

    def get_generic(annotation: Any) -> Any:
        origin = get_origin(annotation)
        if origin is Union:
            return Union
        return _TYPE_MAPPING.get(origin)

    def is_annotated_type(annotation: Any) -> bool:
        return hasattr(annotation, '__metadata__') and hasattr(annotation, '__origin__')



from wunderkafka.serdes.avro.serializers import FastAvroSerializer
from wunderkafka.serdes.avro.deserializers import FastAvroDeserializer
from wunderkafka.serdes.avromodel.serializers import AvroModelSerializer



import json
import re
from typing import Any, Dict, Type
from dataclasses import is_dataclass

from dataclasses_avroschema import AvroModel

from wunderkafka.logger import logger
from wunderkafka.serdes.avromodel.pydantic import derive_from_pydantic, exclude_pydantic_class


def derive(model_type: Type[object], topic: str, *, is_key: bool = False) -> str:
    pydantic_model = derive_from_pydantic(model_type)
    if pydantic_model is None:
        if is_dataclass(model_type):
            # https://github.com/python/mypy/issues/14941
            model_schema = model_type.avro_schema_to_python()                                             # type: ignore
        else:
            # Non-dataclasses objects may allow mixing defaults and non-default fields order,
            # so to still reuse dataclasses_avroschema, we extract fields and reorder them to satisfy dataclasses
            # restrictions, than reorder them back.
            # All these hacks will work only for flat schemas:
            #   we avoid describing objects via nested types for HDFS's sake.
            attributes = _extract_attributes(model_type)
            ordering = list(attributes['__annotations__'])
            crafted_model: Type[AvroModel] = type(model_type.__name__, (AvroModel,), attributes)
            model_schema = crafted_model.avro_schema_to_python()
            fields_map = {field_data['name']: field_data for field_data in model_schema['fields']}
            reordered_fields = [fields_map[attr] for attr in ordering]
            model_schema['fields'] = reordered_fields
    else:
        model_schema = pydantic_model.avro_schema_to_python()
    model_schema.pop('doc', None)
    suffix = 'key' if is_key else 'value'
    if hasattr(model_type, 'Meta') and hasattr(model_type.Meta, 'name'):
        model_schema['name'] = model_type.Meta.name
    else:
        model_schema['name'] = '{0}_{1}'.format(re.sub('[^\w_]','_', topic), suffix)
    if model_schema['name'].startswith('_'):
        logger.warning('Topic name {0} starts with underscore, which is not allowed for Avro schema names.')
        model_schema['name'] = model_schema['name'][1:]
    if pydantic_model:
        # I would not prefer to just override PydanticParser because it always may be broken or thrown-out
        model_schema = exclude_pydantic_class(model_schema)
    return json.dumps(model_schema)


def _extract_attributes(type_: Type[object]) -> Dict[str, Any]:
    fields = vars(type_).get('__annotations__', {})
    _, *parents = type_.mro()
    for base in parents:
        new_fields = {**vars(base).get('__annotations__', {})}
        # Currently it is `model_config: ClassVar[SettingsConfigDict]`
        # https://github.com/pydantic/pydantic-settings/blob/919a20b77527ecc1cd6eeb0a09ca22cc21486fb8/pydantic_settings/main.py#L166
        for field in ['__slots__', 'klass', 'metadata', 'schema_def', '__config__', 'model_config']:
            new_fields.pop(field, None)
        fields = {**new_fields, **fields}

    attributes = {}
    attributes.update({'__annotations__': fields})
    meta = vars(type_).get('Meta', None)
    if meta is not None:
        attributes.update({'Meta': meta})
    return attributes
